package Evo::Test::Call;
use Evo -Class;

has 'args';
has 'exception', optional;
has 'result_fn';

sub result($self) {
  return unless $self->result_fn;
  $self->result_fn->();
}

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Evo::Test::Call

=head1 VERSION

version 0.0403

=head1 AUTHOR

alexbyk.com

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by alexbyk.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
